package cs636.pizza.dao;

// Example JUnit5 test 
// Important: use this import for Test, not the JUnit4 import for Test
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.test.context.junit.jupiter.SpringExtension;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.jdbc.JdbcTest;
import org.springframework.test.context.ContextConfiguration;
import java.sql.SQLException;
import javax.persistence.EntityManagerFactory;
import cs636.pizza.config.PizzaSystemConfig;

//@RunWith(SpringRunner.class)
// Junit5--
@ExtendWith(SpringExtension.class)
//Needed to handle DataSource config
@JdbcTest
//to be minimalistic, configure only the needed beans, avoid using AppConfig
@ContextConfiguration(classes = { DbDAO.class, MenuDAO.class })
//use application-test.properties in src/main/resources instead of application.properties

public class MenuDAOTest {
	@Autowired
	private DbDAO dbDAO;
	@Autowired
	private MenuDAO menuDAO;

	@BeforeEach
	public void setup() {
		EntityManagerFactory emf = PizzaSystemConfig.configureJPA(null, null, null);
		dbDAO.setEntityManagerFactory(emf);
		dbDAO.startTransaction();
		dbDAO.initializeDb(); // no orders, toppings, sizes
		menuDAO = new MenuDAO(dbDAO);
		dbDAO.commitTransaction();
	}

	@AfterEach
	public void tearDown() {
		// This executes even after an exception as well as normal ending
		// so we need to rollback here in case of exception
		// (If the transaction was successful, it's already
		// committed, and this won't hurt.)
		dbDAO.rollbackAfterException();
		dbDAO.shutdown();
	}

	@Test
	public void testCreateTopping() throws SQLException {
		dbDAO.startTransaction();
		menuDAO.createMenuTopping("anchovies");
		int count = menuDAO.findMenuToppings().size();
		dbDAO.commitTransaction();
		Assertions.assertTrue(count == 1, () -> "first topping not created");
	}

	@Test
	public void testDuplicateTopping() {
		// With JPA, have various possible Exceptions, so catch them all
		Assertions.assertThrows(Exception.class, () -> {
			dbDAO.startTransaction();
			menuDAO.createMenuTopping("anchovies");
			menuDAO.createMenuTopping("anchovies");
			dbDAO.commitTransaction();
		});
	}

}
